## Project:     Multivariate Distribution Regression
## Task:        Simulation Studies
## Author:      Jonas Meier
## Date:        21.2.2022
rm(list = ls())

# Specify Path (needs to be changed by user!)
path <- "C:/Users/Jonas/Dropbox/Multivariate DR"

# Packages ----------------------------------------------------------------
library(mvtnorm)
library(parallel)
library(copula)
library(truncnorm)
library(stargazer)
library(VGAM)
library(VineCopula)
library(fastglm)
library(reshape2)
library(plotly)
library(purrr)
library(tidyr)
library(devtools)
library(CondCopulas)
library(expm)
library(akima)
library(Hmisc)
devtools::install_github("AlexisDerumigny/CondCopulas")

# Basic MDR Functions -----------------------------------------------------
source(paste(path,"MDR Functions.R",sep=""))

# Additional Simulation Functions -----------------------------------------
dgp               <- function(n,fm1,fm2,fsd1,fsd2,fcor){
  
  X           <- runif(n)
  cov         <- fcor(X) * fsd1(X) * fsd2(X)
  Sigma       = array(dim = c(2,2,n))
  Sigma[1,1,] <- fsd1(X)^2
  Sigma[2,2,] <- fsd2(X)^2
  Sigma[1,2,] <- Sigma[2,1,] <- cov
  
  SqrtSigma   = apply(X = Sigma, MARGIN = c(3), FUN = sqrtm)
  SqrtSigma   = array(SqrtSigma, dim = c(2,2,n))
  
  Z           = matrix(rnorm(2*n), ncol = 2)
  Y1          = fm1(X) + Z[,1] * SqrtSigma[1,1,] + Z[,2] * SqrtSigma[1,2,]
  Y2          = fm2(X) + Z[,1] * SqrtSigma[2,1,] + Z[,2] * SqrtSigma[2,2,]
  data        <- cbind(Y1,Y2,fm1(X),fm2(X),Sigma[1,1,],Sigma[2,2,],Sigma[1,2,])
  F0          <- apply(data,1,function(x){pmvnorm(lower=-Inf, upper=c(x[1],x[2]), mean=c(x[3],x[4]), sigma=matrix(c(x[5],x[7],x[7],x[6]),2,2))})
  as.data.frame(cbind(Y1,Y2,X,F0))
}
dgp.beta          <- function(reg,beta){
  if(nrow(beta[1,,])!=ncol(beta[1,,])){
    stop("beta needs to be an array of square matrices")
  }
  if(is.matrix(reg)==F){
    stop("reg needs to be a matrix")
  }
  
  thres <- nrow(beta[1,,])
  index <- array(0,dim=c(nrow(reg),thres,thres))
  fit   <- array(0,dim=c(nrow(reg),thres,thres))
  for(i in 1:nrow(reg)){
    for(j in 1:(ncol(reg))){
      index[i,,] <- index[i,,] + reg[i,j]*beta[j,,]
    }
    fit[i,,] <- 1-exp(-exp(index[i,,]))
  }
  
  u       <- runif(nrow(reg))
  y       <- matrix(NA,nrow(reg),2)
  for(i in 1:nrow(reg)){
    y[i,] <- arrayInd(which.min(abs(u[i]-fit[i,,])), dim(fit[i,,]))
  }
  y <- y/thres
  return(y)
}
CMSE              <- function(d,estF,grid){
  
  # data preparation
  d       <- cbind(d,estF)
  qy1     <- quantile(d$Y1,grid,na.rm = TRUE)
  qy2     <- quantile(d$Y2,grid,na.rm = TRUE)
  d$t1    <- apply(as.matrix(d$Y1),1,findt,Q=qy1)
  d$t2    <- apply(as.matrix(d$Y2),1,findt,Q=qy2)
  gr      <- expand.grid(qy1,qy2)
  gr$id   <- seq(1,nrow(gr),1)
  colnames(gr) <- c("t1", "t2", "id")
  d       <- merge(d,gr)
  d       <- na.omit(d)
  mestF   <- cbind(tapply(d$id,d$id,mean),tapply(d$estF,d$id,mean),tapply(d$F0,d$id,mean))
  colnames(mestF) <- c("id","mestF","mtrue_F")
  d       <- merge(d,mestF)
  
  # statistics
  OMSE    <- mean( ((d$estF - d$F0)^2) ) 
  OCMSE   <- mean( ((d$estF - d$F0)^2) / (d$F0*(1-d$F0)) ) 
  MSE     <- by(d,d$id,function(x){mean( ((x$estF - x$mtrue_F)^2) )})
  VAR     <- by(d,d$id,function(x){mean( ((x$estF - x$mestF)^2) )})
  BIAS    <- by(d,d$id,function(x){((x$mestF-x$mtrue_F)^2)[1]})
  res     <- cbind(MSE,VAR,BIAS)
  res     <- res[res[,1]!=Inf,]
  CMSE    <- by(d,d$id,function(x){mean( ((x$estF - x$mtrue_F)^2)/ (x$mtrue_F*(1-x$mtrue_F)) )})
  CVAR    <- by(d,d$id,function(x){mean( ((x$estF - x$mestF)^2) / (x$mtrue_F*(1-x$mtrue_F)))})
  CBIAS   <- by(d,d$id,function(x){((x$mestF-x$mtrue_F)^2)[1]/(x$mtrue_F*(1-x$mtrue_F))[1]})
  Cres    <- cbind(CMSE,CVAR,CBIAS)
  Cres    <- Cres[Cres[,1]!=Inf,]
  output  <- c(c(apply(res,2,mean,na.rm=T)),c(apply(Cres,2,mean,na.rm=T)),OMSE,OCMSE)
  names(output)  <- c("MSE","VAR","BIAS","CMSE","CVAR","CBIAS","OMSE","OCMSE")
  output
}
findt             <- function(x,Q){
  n <- as.numeric(length(Q))
  t <- Q[1]
  for(i in 1:n){
    if(x>Q[i]){t <- Q[i+1]}
    else{end}
  }
  t
}
Fatvalue_Cop      <- function(YX,pX,psY1,psY2,CDF){
  loc_y1 <- which.min(abs(YX[1]-psY1))
  loc_y2 <- which.min(abs(YX[2]-psY2))
  loc_x  <- which.min(abs(YX[3]-pX))
  FatX   <- CDF[loc_x,,][loc_y1,loc_y2]
  FatX
}
np_copula         <- function(dep,reg,grid,h,npkernel="Gaussian"){
  predX   <- quantile(reg,grid,na.rm = TRUE)
  NP_cop  <- estimateNPCondCopula(dep[,1],dep[,2],reg,grid,grid,predX,kernel=npkernel,h=h)
  NP_cop  <- aperm(NP_cop, c(3,1,2))
  triple  <- cbind(pobs(cbind(dep[,1],dep[,2])),reg)
  F_CNC   <- apply(triple,1,Fatvalue_Cop,predX,grid,grid,NP_cop)
}
p_copula          <- function(dep,reg,grid,h,pfamily=1){
  pseudo  <- as.data.frame(pobs(cbind(dep[,1],dep[,2],reg)))
  P_cop   <- estimateParCondCopula(pseudo$V1,pseudo$V2,pseudo$reg,grid,family = pfamily,h=h)
  qX      <- unlist(lapply(pseudo$reg,function(x,g){which.min(abs(x-g))},g=grid))
  parX    <- P_cop[qX]
  index   <- seq(1,nrow(dep),1)
  indexnonmissing <- which(!is.na(parX))
  nmparX  <- parX[is.na(parX)==0] 
  nmY1    <- pseudo$V1[is.na(parX)==0] 
  nmY2    <- pseudo$V2[is.na(parX)==0] 
  nmCDF   <- VineCopula::BiCopCDF(nmY1, nmY2, family = pfamily, par = nmparX)
  nmCDF   <- as.data.frame(cbind(indexnonmissing,nmCDF))
  colnames(nmCDF)[1] <- "index"
  all     <- as.data.frame(cbind(index,matrix(NA,nrow(dep))))
  F_CPC   <- merge(all,nmCDF,all=TRUE)[,3]
}
sim               <- function(size,GR,h_np,h_p,fmy1=function(x){0},fmy2=function(x){0},
                              fsdy1=function(x){1},fsdy2=function(x){1},fcor,
                              link="cloglog",npkernel="Gaussian",pfamily=1){
  
  # Draw Data
  data    <- dgp(size,fmy1,fmy2,fsdy1,fsdy2,fcor)
  
  # MDR
  fitMDR  <- mdr(cbind(data$Y1,data$Y2),as.matrix(data$X),quantiles=GR,se=F,parallel=F,link=link)
  F_MDR   <- as.numeric(fitMDR$C.CDF)
  
  # Unconditional NP Copula
  #fitUNC  <- fitCopula(normalCopula(NA),pobs(as.matrix(data[,1:2])))
  #F_UNC   <- pCopula(pobs(cbind(data$Y1,data$Y2)),fitUNC@copula)
  
  # Conditional NP Copula
  F_NC   <- np_copula(cbind(data$Y1,data$Y2),data$X,GR,h_np,npkernel=npkernel)
  
  # Conditional P Copula
  F_PC   <- p_copula(cbind(data$Y1,data$Y2),data$X,GR,h_p,pfamily=pfamily)
  
  # MSE, Variance and Bias
  MSE_MDR <- CMSE(data,F_MDR,GR)
  MSE_NC  <- CMSE(data,F_NC,GR)
  MSE_PC  <- CMSE(data,F_PC,GR)
  output  <- rbind(MSE_MDR,MSE_NC,MSE_PC)
  rownames(output) <- c("MDR", "NP", "PC")
  output 
}
dgp.beta          <- function(size,regtype,beta){
  if(nrow(beta[1,,])!=ncol(beta[1,,])){
    stop("beta needs to be an array of square matrices")
  }
  if(regtype=="cons"){reg <- as.matrix(rep(1,size))}
  if(regtype=="unif"){reg <- cbind(1,runif(size))}
  
  thres <- nrow(beta[1,,])
  index <- array(0,dim=c(nrow(reg),thres,thres))
  fit   <- array(0,dim=c(nrow(reg),thres,thres))
  for(i in 1:nrow(reg)){
    for(j in 1:(ncol(reg))){
      index[i,,] <- index[i,,] + reg[i,j]*beta[j,,]
    }
    fit[i,,] <- 1-exp(-exp(index[i,,]))
  }
  
  # draw y1 according to marginal
  cdfy1   <- apply(fit[,,],1:2,max)
  u1      <- runif(size)
  y1      <- rowSums(cdfy1<u1)
  y1[y1==0] <- 1
  
  # draw y2 according to conditional cdf
  cdfy2 <- array(NA,dim=c(nrow(reg),thres,thres))
  for(i in 1:nrow(reg)){
    for(j in 2:(thres-1)){
      cdfy2[i,j,]  <- (fit[i,,j+1]-fit[i,,j-1])/(fit[i,thres,j+1]-fit[i,thres,j-1])
    }
    cdfy2[i,1,]     <- cdfy2[i,2,]
    cdfy2[i,thres,] <- cdfy2[i,thres-1,]
  }
  cdfy2[cdfy2 > 1] <- 1
  
  ccdfy2 <- matrix(NA,nrow(reg),thres)
  for(i in 1:nrow(reg)){
    ccdfy2[i,] <- cdfy2[i,y1[i],]
  }
  u2      <- runif(size)
  y2      <- rowSums(ccdfy2<u2)
  y2[y2==0] <- 1
  y <- cbind(y1,y2)/thres
  return(list(reg,y))
}
sim.beta          <- function(size,regtype,beta,ngridval){
  
  # draw data
  data <- dgp.beta(size,regtype,beta)
  ygrid <- seq(1,ngridval,1)/ngridval
  ygrid <- cbind(ygrid,ygrid)
  
  # Estimation
  mdrfit      <- mdr(data[[2]],as.matrix(data[[1]]),cons=F,parallel=F,
                     se=F,fixedval=ygrid)
  coef        <- cbind(mdrfit$Grid,mdrfit$Coef)
  return(list(coef,data))
}
mpdf              <- function(mcdf){
  mcdf <- setNames(melt(mcdf), c('x', 'y', 'z'))
  mcdf$x <- mcdf$x/length(unique(mcdf$x))
  mcdf$y <- mcdf$y/length(unique(mcdf$y))
  df_x <- unique(mcdf$x)
  df_y <- unique(mcdf$y)
  
  # first derivative
  dx <- apply(mcdf, 1, function(x) mF_dx(x["x"], x["y"], data=mcdf, uval=df_x))
  #dy <- apply(mcdf, 1, function(x) mF_dy(x["x"], x["y"], data=mcdf, uval=df_y))
  mcdf <- cbind(mcdf,dx)
  colnames(mcdf) <- c("x","y","F_XY","F_dx")
  
  # cross derivative
  dxdy <- apply(mcdf, 1, function(x) mF_dxdy(x["x"], x["y"], data=mcdf, uval=df_y))
  dxdy[is.na(dxdy)] <- dxdy[dxdy < 0] <- 0
  mcdf <- cbind(mcdf,dxdy)
  colnames(mcdf) <- c("x","y","F_XY","F_dx","mpdf")
  mcdf
}
results_beta      <- function(results,ngridval,reps,beta,dimbeta,thres,qy){
  
  # sorting
  res         <- expand.grid(seq(1,ngridval,1)/ngridval,seq(1,ngridval,1)/ngridval)
  dataq       <- array(NA,dim=c(reps,length(qy),2))
  colnames(res) <- c("y1", "y2")
  true        <- melt(beta[dimbeta,,])
  true$Var1   <- true$Var1/thres
  true$Var2   <- true$Var2/thres
  names(true) <- c("y1", "y2", "tcoef")
  res         <- merge(res,true)
  
  # merge results
  for(i in 1:reps){
    temp        <- results[[i]][[1]]
    temp        <- cbind(temp[,1:2],temp[,2+dimbeta])
    names(temp) <- c("y1", "y2", paste0("S",i))
    res         <- merge(res,temp,all=T)
    dataq[i,,]  <- apply(results[[i]][[2]][[2]],2,quantile,probs=qy)
  }
  
  # evaluation matrix
  mqval   <- apply(dataq,2:3,mean)
  valy1   <- res[apply(as.matrix(mqval[,1]),1,function(x) which.min(abs(x-res[,1]))),1]
  valy2   <- res[apply(as.matrix(mqval[,2]),1,function(x) which.min(abs(x-res[,2]))),2]
  valy    <- expand.grid(valy1,valy2)
  colnames(valy) <- c("y1","y2")
  eval    <- as.matrix(merge(res,valy))
  
  # MSE
  mse  <- (eval[,4:ncol(eval)]-eval[,3])^2
  mse  <- apply(mse,1,mean)
  bias <- (apply(eval[,4:ncol(eval)],1,mean)-eval[,3])^2
  var  <- apply((eval[,4:ncol(eval)]-apply(eval[,4:ncol(eval)],1,mean))^2,1,mean)
  mse  <- cbind(mse,var,bias)
  return(list(mse,eval))
}

# Parameters --------------------------------------------------------------
# All Simulations 
samplesizes <- c(400,1600,6400)                           # Sample sizes
n_clus      <- 48                                         # number of clusters
nreps       <- 10000                                      # number of replications

# Simulation 1 and 2
gridv       <- 10                                         # Fixed grid values for each outcome (data is simulated to lie between 0 and 1)
yqval       <- c(.25,.5,.75)                              # Quantiles at which Beta should be evaluated
nthres      <- 100                                        # Grid for true Coefficients
trueb1      <- array(NA,dim=c(1,nthres,nthres))           # True Regression Coefficients, Simulation 1
trueb2      <- array(NA,dim=c(2,nthres,nthres))           # True Regression Coefficients, Simulation 2
for(i in 1:nthres){
  for(j in 1:nthres){
    trueb1[1,i,j] <- i*.025+j*.025+sqrt(i*j)*0.05-6.5
    trueb2[1,i,j] <- i*.025+j*.025+sqrt(i*j)*0.05-6.5
    trueb2[2,i,j] <- 2-i^2*0.0002-j^2*0.0002
    #trueb2[2,i,j] <- (-(i-nthres/2)^3*.00001+(j-nthres/4)^2*.0005+2)/6
    #trueb2[2,i,j] <- (i-nthres/2)^2*.001+(j-nthres/2)^2*.001-2.5
  }
}
res1        <- vector(mode = "list", length(samplesizes))
res2c       <- vector(mode = "list", length(samplesizes))
res2b       <- vector(mode = "list", length(samplesizes))
time1       <- rep(NA,length(samplesizes))
time2       <- rep(NA,length(samplesizes))

# Simulation 3 and 4
grid        <- seq(from=0.001, to=.999, by=.998/(25-1))   # Grid for estimation quantiles. MDR: outcome grid. NPC: pseudo observations for Y1 and Y2 and prediction quantiles of X. PC: pseudo observations for X.
h_np        <- 0.15                                       # bandwidth nonparametric copula model
h_p         <- 0.15                                       # bandwidth parametric copula model
modelspec   <- c("m1","m2","m3")
n_misspec   <- 4
res3        <- array(NA,dim=c(length(samplesizes),length(modelspec),3,8))
res4        <- array(NA,dim=c(length(samplesizes),n_misspec,3,8))
time3       <- matrix(NA,length(samplesizes),length(modelspec))
time4       <- matrix(NA,length(samplesizes),n_misspec)


# Simulation 1: Consistency of Beta (only a constant) ---------------------
k <- 1
for(s in samplesizes){
  start.time <- Sys.time()
  n_clus  <- detectCores()
  cl      <- makeCluster(n_clus)
  clusterEvalQ(cl,{library(reshape2)
    library(fastglm)
    library(parallel)})
  clusterExport(cl,c("sim.beta","mdr","gridloc","singleDR","parDRBS","subsample","count_cdf","linkf","dgp.beta"))
  mdrfit       <- clusterApply(cl, 1:nreps, function(i,size,regtype,beta,ngridval) sim.beta(size,regtype,beta,ngridval),
                               size=s, regtype="cons", beta=trueb1, ngridval=gridv)
  stopCluster(cl)
  closeAllConnections()
  end.time <- Sys.time()
  time.taken <- end.time - start.time
  time.taken
  
  res1[[k]] <- results_beta(mdrfit,gridv,nreps,trueb1,1,nthres,yqval)
  time1[k]  <- round(difftime(end.time, start.time, units = "secs"))
  print(c(s,round(time1[k])))
  k <- k+1
}

# Table
mse1    <- format(round(rbind(t(res1[[1]][[1]]),t(res1[[2]][[1]]),t(res1[[3]][[1]])),4),nsmall=4)
dsizes  <- c(400,NA,NA,1600,NA,NA,6400,NA,NA)
crit    <- rep(c("MSE","VAR","BIAS"),3)
names   <- c("N","25th,25th","25th,50th","25th,75th","50th,25th","50th,50th","50th,75th","75th,25th","75th,50th","75th,75th")
mse1    <- cbind(dsizes,crit,mse1)

# Output into Latex
stargazer(mse1, summary = F, rownames = F, covariate.labels = names,
          font.size = "small", title="Simulation: IMSE, IVAR and IBIAS", label = "t:simulation",
          out = "C://Users//mapo//Dropbox//Jonas//Simulation Results MDR//sim_b_conly.tex")

# Graph
eval <- res1[[2]][[2]]
setwd(paste(path,"/3 Results/Simulation",sep=""))
pdf('normality_beta_1600conly.pdf',height=9,width=9)
par(mfrow=c(3,3), cex=.8)
# DGP 1
# Y1 25th, Y2 25th
hist(eval[1,4:ncol(eval)],15,col="dodgerblue2", main="Y1 25th, Y2 25th", xlab="Est. Coef.")
abline(v=eval[1,3],col="red",lwd=3)
# Y1 25th, Y2 Median
hist(eval[2,4:ncol(eval)],15,col="dodgerblue2", main="Y1 25th, Y2 Median", xlab="Est. Coef.")
abline(v=eval[2,3],col="red",lwd=3)
# Y1 25th, Y2 75th
hist(eval[3,4:ncol(eval)],15,col="dodgerblue2", main="Y1 25th, Y2 75th", xlab="Est. Coef.")
abline(v=eval[3,3],col="red",lwd=3)

# Y1 Median, Y2 25th
hist(eval[4,4:ncol(eval)],15,col="dodgerblue2", main="Y1 Median, Y2 25th", xlab="Est. Coef.")
abline(v=eval[4,3],col="red",lwd=3)
# Y1 Median, Y2 Median
hist(eval[5,4:ncol(eval)],15,col="dodgerblue2", main="Y1 Median, Y2 Median", xlab="Est. Coef.")
abline(v=eval[5,3],col="red",lwd=3)
# Y1 Median, Y2 75th
hist(eval[6,4:ncol(eval)],15,col="dodgerblue2", main="Y1 Median, Y2 75th", xlab="Est. Coef.")
abline(v=eval[6,3],col="red",lwd=3)

# Y1 75th, Y2 Median
hist(eval[7,4:ncol(eval)],15,col="dodgerblue2", main="Y1 75th, Y2 25th", xlab="Est. Coef.")
abline(v=eval[7,3],col="red",lwd=3)
# Y1 75th, Y2 25th
hist(eval[8,4:ncol(eval)],15,col="dodgerblue2", main="Y1 75th, Y2 Median", xlab="Est. Coef.")
abline(v=eval[8,3],col="red",lwd=3)
# Y1 75th, Y2 75th
hist(eval[9,4:ncol(eval)],15,col="dodgerblue2", main="Y1 75th, Y2 75th", xlab="Est. Coef.")
abline(v=eval[9,3],col="red",lwd=3)
dev.off()


# Simulation 2: Consistency of Beta, Uniform Regressor --------------------
k <- 1
for(s in samplesizes){
  start.time <- Sys.time()
  n_clus  <- detectCores()
  cl      <- makeCluster(n_clus)
  clusterEvalQ(cl,{library(reshape2)
    library(fastglm)
    library(parallel)})
  clusterExport(cl,c("sim.beta","mdr","gridloc","singleDR","parDRBS","subsample","count_cdf","linkf","dgp.beta"))
  mdrfit       <- clusterApply(cl, 1:nreps, function(i,size,regtype,beta,ngridval) sim.beta(size,regtype,beta,ngridval),
                               size=s, regtype="unif", beta=trueb2, ngridval=gridv)
  stopCluster(cl)
  closeAllConnections()
  end.time <- Sys.time()
  time.taken <- end.time - start.time
  time.taken
  
  res2c[[k]]  <- results_beta(mdrfit,gridv,nreps,trueb2,1,nthres,yqval)
  res2b[[k]]  <- results_beta(mdrfit,gridv,nreps,trueb2,2,nthres,yqval)
  time2[k]    <- round(difftime(end.time, start.time, units = "secs"))
  print(c(s,round(time2[k])))
  k <- k+1
}

# Table
mse2c     <- rbind(t(res2c[[1]][[1]]),t(res2c[[2]][[1]]),t(res2c[[3]][[1]]))
mse2b     <- rbind(t(res2b[[1]][[1]]),t(res2b[[2]][[1]]),t(res2b[[3]][[1]]))
mse2c[2:9,] <- sqrt(mse2c[2:9,]) # standard deviation, not variance; and not bias squared
mse2b[2:9,] <- sqrt(mse2b[2:9,]) # standard deviation, not variance; and not bias squared
mse2c     <- format(round(mse2c,4),nsmall=4)
mse2b     <- format(round(mse2b,4),nsmall=4)

dsizes    <- c(400,NA,NA,1600,NA,NA,6400,NA,NA)
crit      <- rep(c("MSE","SD","Bias"),3)
names     <- c("N","25th,25th","25th,50th","25th,75th","50th,25th","50th,50th","50th,75th","75th,25th","75th,50th","75th,75th")
mse2c     <- cbind(dsizes,crit,NA,mse2c)
mse2b     <- cbind(dsizes,crit,NA,mse2b)

# Output into Latex
setwd(paste(path,"/5_Paper/PaperTables",sep=""))
cat(stargazer(mse2c, float=FALSE, rownames = F)[9:17], file="sim_b_c.tex", sep="\n")
cat(stargazer(mse2b, float=FALSE, rownames = F)[9:17], file="sim_b_b.tex", sep="\n")

save.image("C://Users//mapo//Dropbox//Jonas//Simulation Results MDR//simres_b.RData")
load("C://Users//jonas//Dropbox//Martina//Simulation Results MDR//simres_b.RData")


# Graphs
setwd(paste(path,"/3 Results/Simulation",sep=""))
eval <- res2b[[2]][[2]]

# Y1 25th, Y2 25th
pdf('beta_1600b_y1_25_y2_25.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[1,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[1,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 25th, Y2 Median
pdf('beta_1600b_y1_25_y2_50.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[2,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[2,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 25th, Y2 75th
pdf('beta_1600b_y1_25_y2_75.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[3,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[3,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 Median, Y2 25th
pdf('beta_1600b_y1_50_y2_25.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[4,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[4,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 Median, Y2 Median
pdf('beta_1600b_y1_50_y2_50.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[5,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[5,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 Median, Y2 75th
pdf('beta_1600b_y1_50_y2_75.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[6,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[6,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 75th, Y2 Median
pdf('beta_1600b_y1_75_y2_25.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[7,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[7,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 75th, Y2 25th
pdf('beta_1600b_y1_75_y2_50.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[8,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[8,3],col="black",lwd=5, lty=2)
dev.off()

# Y1 75th, Y2 75th
pdf('beta_1600b_y1_75_y2_75.pdf',height=7,width=7)
par(mai=c(1,1,.2,.2))
hist(eval[9,4:ncol(eval)],50,col="deepskyblue3", border="deepskyblue3", main="", xlab="Estimated Coefficient", cex.lab=2, cex.axis=2, freq = FALSE)
abline(v=eval[9,3],col="black",lwd=5, lty=2)
dev.off()

# Graph of Coefficients
library(RColorBrewer)
png('cons_dist.png',type="cairo", width = 1000, height = 900)
par(mar = c(5,6,4,2))
filled.contour(trueb2[1,,],color.palette = colorRampPalette(brewer.pal(9,"Blues")),
               plot.axes={axis(1,cex.axis=2.5)
                 axis(2,cex.axis=2.5)},
               key.axes = axis(4,cex.axis=2.5), 
               plot.title={title(xlab="Y1",cex.lab=2.5)
                 title(ylab="Y2",cex.lab=2.5)})
dev.off()

png('beta_dist.png',type="cairo", width = 1000, height = 900)
par(mar = c(5,6,4,2))
filled.contour(trueb2[2,,],color.palette = colorRampPalette(brewer.pal(9,"Blues")),
               plot.axes={axis(1,cex.axis=2.5)
                 axis(2,cex.axis=2.5)},
               key.axes = axis(4,cex.axis=2.5), 
               plot.title={title(xlab="Y1",cex.lab=2.5)
                 title(ylab="Y2",cex.lab=2.5)})
dev.off()

png('mcdf_x0.png',type="cairo", width = 1000, height = 900)
par(mar = c(5,6,4,2))
filled.contour(1-exp(-exp(trueb1[1,,])),color.palette = colorRampPalette(brewer.pal(9,"Blues")),
               plot.axes={axis(1,cex.axis=2.5)
                 axis(2,cex.axis=2.5)},
               key.axes = axis(4,cex.axis=2.5), 
               plot.title={title(xlab="Y1",cex.lab=2.5)
                 title(ylab="Y2",cex.lab=2.5)})
dev.off()

png('mcdf_x1.png',type="cairo", width = 1000, height = 900)
par(mar = c(5,6,4,2))
filled.contour(1-exp(-exp(trueb2[1,,]+trueb2[2,,])),color.palette = colorRampPalette(brewer.pal(9,"Blues")),
               plot.axes={axis(1,cex.axis=2.5)
                 axis(2,cex.axis=2.5)},
               key.axes = axis(4,cex.axis=2.5), 
               plot.title={title(xlab="Y1",cex.lab=2.5)
                 title(ylab="Y2",cex.lab=2.5)})
dev.off()


# # Output into Latex
# stargazer(mse2c, summary = F, rownames = F, covariate.labels = names,
#           font.size = "small", title="Simulation: IMSE, IVAR and IBIAS", label = "t:simulation",
#           out = "C://Users//Jonas//Dropbox//Multivariate DR//3 Results//Simulation//sim_b_c.tex")
# 
# # Output into Latex
# stargazer(mse2b, summary = F, rownames = F, covariate.labels = names,
#           font.size = "small", title="Simulation: IMSE, IVAR and IBIAS", label = "t:simulation",
#           out = "C://Users//Jonas//Dropbox//Multivariate DR//3 Results//Simulation//sim_b_b.tex")
# 
# save.image("C://Users//Jonas//Dropbox//Multivariate DR//3 Results//SHP//Simulation Results//simres_b.RData")




# Simulation 3: Comparison MDR and Copulas --------------------------------
i <- j <- 1
for(s in samplesizes){
  for(f in modelspec){
    start.time <- Sys.time()
    cl <- makeCluster(n_clus)
    clusterEvalQ(cl,{library(mvtnorm)
      library(VineCopula)
      library(expm)
      library(fastglm)
      library(CondCopulas)})
    clusterSetRNGStream(cl, iseed = 12345678) 
    clusterExport(cl,c("singleDR","linkf","countmat","Fatvalue","gridloc","mdr","count_cdf",
                       "sim","dgp","CMSE","findt","Fatvalue_Cop","np_copula","p_copula"))
    
    if(j==1){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
      sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
      size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
      fsdy1=function(x){1},fsdy2=function(x){1},fcor=function(x){sin(4*pi*x)},link="cloglog",npkernel="Gaussian",pfamily=1)
    }
    if(j==2){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
      sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
      size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
      fsdy1=function(x){(x+.378)^2},fsdy2=function(x){(x+.378)^2},fcor=function(x){sin(4*pi*x)},link="cloglog",npkernel="Gaussian",pfamily=1)
    }
    if(j==3){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
      sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
      size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
      fsdy1=function(x){(x+.273)^4},fsdy2=function(x){(x+.273)^4},fcor=function(x){sin(4*pi*x)},link="cloglog",npkernel="Gaussian",pfamily=1)
    }
    res3[i,j,,] <- apply(simplify2array(simres), 1:2, mean, na.rm=T)
    rownames(res3[i,j,,]) <- c("MDR", "NP", "PC")
    colnames(res3[i,j,,]) <- c("MSE","VAR","BIAS","CMSE","CVAR","CBIAS","OMSE","OCMSE")
    
    stopCluster(cl)
    closeAllConnections()
    end.time <- Sys.time()
    time3[i,j] <- round(difftime(end.time, start.time, units = "secs"))
    print(c(s,f,round(time3[i,j])))
    j <- j+1
  }
  j <- 1
  i <- i+1
}


3load("C://Users//jmeier//Dropbox//Martina//Simulation Results MDR//simres.RData")
#res3 <- res
# Table
mse3 <- cbind(rbind(res3[1,1,,1:3],res3[2,1,,1:3],res3[3,1,,1:3]),
                           rbind(res3[1,2,,1:3],res3[2,2,,1:3],res3[3,2,,1:3]),
                           rbind(res3[1,3,,1:3],res3[2,3,,1:3],res3[3,3,,1:3]))
cmse3 <- cbind(rbind(res3[1,1,,4:6],res3[2,1,,4:6],res3[3,1,,4:6]),
                            rbind(res3[1,2,,4:6],res3[2,2,,4:6],res3[3,2,,4:6]),
                            rbind(res3[1,3,,4:6],res3[2,3,,4:6],res3[3,3,,4:6]))
mse3[,c(2,3,5,6,8,9)]   <- sqrt(mse3[,c(2,3,5,6,8,9)]) # standard deviation, not variance; and not bias squared
cmse3[,c(2,3,5,6,8,9)]  <- sqrt(cmse3[,c(2,3,5,6,8,9)]) # standard deviation, not variance; and not bias squared
mse3              <- format(round(mse3,4),nsmall=4)
cmse3             <- format(round(cmse3,4),nsmall=4)

names   <- c("N","Model","MSE","SD","Bias","MSE","SD","Bias","MSE","SD","Bias")
model   <- rep(c("MDR","NC","PC"),3)
dsizes  <- c(400,NA,NA,1600,NA,NA,6400,NA,NA)

mse3  <- cbind(dsizes,model,mse3)
cmse3 <- cbind(dsizes,model,cmse3)

# Output into Latex
setwd(paste(path,"/5_Paper/PaperTables",sep=""))
cat(stargazer(mse3, float=FALSE, rownames = F)[9:17], file="sim_cdf.tex", sep="\n")
cat(stargazer(cmse3, float=FALSE, rownames = F)[9:17], file="sim_cdf_c.tex", sep="\n")


#save.image("C://Users//Jonas//Dropbox//Multivariate DR//3 Results//SHP//Simulation Results//simres.RData")
save.image("C://Users//mapo//Dropbox//Jonas//Simulation Results MDR//simres.RData")

sum(time3)/3600

# Simulation 4: Comparison MDR and Copulas under Misspecification ---------
i <- 1
for(s in samplesizes){
  for(j in 1:n_misspec){
    start.time <- Sys.time()
    cl <- makeCluster(n_clus)
    clusterEvalQ(cl,{library(mvtnorm)
      library(VineCopula)
      library(expm)
      library(fastglm)
      library(CondCopulas)})
    clusterSetRNGStream(cl, iseed = 12345678) 
    clusterExport(cl,c("singleDR","linkf","countmat","Fatvalue","gridloc","mdr","count_cdf",
                       "sim","dgp","CMSE","findt","Fatvalue_Cop","np_copula","p_copula"))
    
    if(j==1){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
        sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
        size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
        fsdy1=function(x){(x+.273)^4},fsdy2=function(x){(x+.273)^4},fcor=function(x){sin(4*pi*x)},
        link="cloglog",npkernel="Gaussian",pfamily=1)
    }
    if(j==2){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
        sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
        size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
        fsdy1=function(x){(x+.273)^4},fsdy2=function(x){(x+.273)^4},fcor=function(x){sin(4*pi*x)},
        link="logit",npkernel="Triangular",pfamily=4)
    }
    if(j==3){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
        sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
        size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
        fsdy1=function(x){(x+.273)^4},fsdy2=function(x){(x+.273)^4},fcor=function(x){sin(4*pi*x)},
        link="probit",npkernel="Epanechnikov",pfamily=5)
    }
    if(j==4){
      simres    <- clusterApply(cl,1:nreps, function(i,size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily)
        sim(size,GR,h_np,h_p,fmy1,fmy2,fsdy1,fsdy2,fcor,link,npkernel,pfamily),
        size=s,GR=grid,h_np=h_np,h_p=h_p,fmy1=function(x){0},fmy2=function(x){0},
        fsdy1=function(x){(x+.273)^4},fsdy2=function(x){(x+.273)^4},fcor=function(x){sin(4*pi*x)},
        link="linear",npkernel="Epanechnikov",pfamily=6)
    }
    res4[i,j,,] <- apply(simplify2array(simres), 1:2, mean, na.rm=T)
    
    stopCluster(cl)
    closeAllConnections()
    end.time <- Sys.time()
    time4[i,j] <- round(difftime(end.time, start.time, units = "secs"))
    print(c(s,j,round(time4[i,j])))
  }
  i <- i+1
}

load("C://Users//jonas//Dropbox//Martina//Simulation Results MDR//simresm.RData")

# Table
mse4 <- cbind(rbind(res4[1,1,,1:3],res4[2,1,,1:3],res4[3,1,,1:3]),
              rbind(res4[1,2,,1:3],res4[2,2,,1:3],res4[3,2,,1:3]),
              rbind(res4[1,3,,1:3],res4[2,3,,1:3],res4[3,3,,1:3]))
cmse4 <- cbind(rbind(res4[1,1,,4:6],res4[2,1,,4:6],res4[3,1,,4:6]),
               rbind(res4[1,2,,4:6],res4[2,2,,4:6],res4[3,2,,4:6]),
               rbind(res4[1,3,,4:6],res4[2,3,,4:6],res4[3,3,,4:6]))
mse4[,c(2,5,8)]   <- sqrt(mse4[,c(2,5,8)]) # standard deviation, not variance; and not bias squared
cmse4[,c(2,5,8)]  <- sqrt(cmse4[,c(2,5,8)]) # standard deviation, not variance; and not bias squared
mse4              <- format(round(mse4,4),nsmall=4)
cmse4             <- format(round(cmse4,4),nsmall=4)

names   <- c("N","Model","MSE","SD","Bias","MSE","SD","Bias","MSE","SD","Bias")
model   <- rep(c("MDR","NC","PC"),3)
dsizes  <- c(400,NA,NA,1600,NA,NA,6400,NA,NA)

mse4  <- cbind(dsizes,model,mse4)
cmse4 <- cbind(dsizes,model,cmse4)

# Output into Latex
setwd(paste(path,"/5_Paper/PaperTables",sep=""))
cat(stargazer(mse4, float=FALSE, rownames = F)[9:17], file="sim_miss_cdf.tex", sep="\n")
cat(stargazer(cmse4, float=FALSE, rownames = F)[9:17], file="sim_miss_cdf_c.tex", sep="\n")


#save.image("C://Users//Jonas//Dropbox//Multivariate DR//3 Results//SHP//Simulation Results//simresm.RData")
save.image("C://Users//mapo//Dropbox//Jonas//Simulation Results MDR//simresm.RData")

sum(time4)/3600


#c("Correctly Specified","Gumbel","Frank","Joe")
# 
# 
# a <- seq(0,1,.01)
# b <- .5+a
# c <- 2*a
# d <- a^2+.665
# e <- (a+.457)^2
# f <- a^3+.75
# g <- (a+.415)^3
# 
# data <- cbind(a,b,c,d,e,f,g)
# summary(data)
# 
# plot(a,b,type="l",ylim=c(0,3),col="blue")
# lines(a,c,col="green")
# lines(a,d,col="red")
# lines(a,e,col="orange")
# lines(a,f,col="cyan")
# lines(a,g,col="darkblue")
# 
# plot(a,b,type="l",ylim=c(0,3),col="blue")
# lines(a,e,col="orange")
# lines(a,g,col="darkblue")
# 
# a <- seq(0,1,.01)
# mean((a+.273)^8)
# 
# 
# simdata   <- dgp(1000,function(x){0},function(x){0},function(x){(x+.457)},function(x){(x+.457)},function(x){sin(4*pi*x)})
# var(simdata$Y1)
# var(simdata$Y2)
# 
# simdata   <- dgp(1000,function(x){0},function(x){0},function(x){(x+.378)^2},function(x){(x+.378)^2},function(x){sin(4*pi*x)})
# var(simdata$Y1)
# var(simdata$Y2)
# 
# simdata   <- dgp(1000,function(x){0},function(x){0},function(x){(x+.318)^3},function(x){(x+.318)^3},function(x){sin(4*pi*x)})
# var(simdata$Y1)
# var(simdata$Y2)
# 
# simdata   <- dgp(1000,function(x){0},function(x){0},function(x){(x+.273)^4},function(x){(x+.273)^4},function(x){sin(4*pi*x)})
# var(simdata$Y1)
# var(simdata$Y2)
# 
# 
# simdata   <- dgp(1000,function(x){0},function(x){0},function(x){(x+.457)},function(x){(x+.457)},function(x){sin(4*pi*x)})
# mdrfit    <- mdr(cbind(simdata$Y1,simdata$Y2),as.matrix(simdata$X),quantiles=c(.001,seq(.025,.975,.05),.999))
# 
# 
# 
# stats     <- mdr.functionals(mdrfit,cbind(simdata$Y1,simdata$Y2))
# stats[[1]]$mean.var.x
# stats[[1]]$mean.var.y



